// ignore_for_file: unrelated_type_equality_checks

import 'package:flutter_test/flutter_test.dart';

import 'package:object_reference/object_reference.dart';

void main() {
  test("Check reference equality.", () {
    final r1 = "nona".ref();
    final r2 = "nona".ref();
    final rc1 = r1.constRef();

    // false, because r1 and r2 hold an Object with the same
    // String content, but r1 and r2 are not references to
    // the same String Object.
    expect(r1 == r2, false);

    // false, because r1 and r2 hold an Object with the same
    // String content, but r1 and r2 are not references to
    // the same String Object.
    expect(r1.hashCode == r2.hashCode, false);

    // true, because r1 and r2 hold an Object with the same
    // String content. Here it does not matter that r1 and r2
    // are not references to the same String Object, because
    // we are only checking if the Strings have the same content.
    expect(r1.value == r2.value, true);

    // true, because r1 and rc1 are a reference to the same
    // String Object. It does not matter that r1 is a MutableRef
    // and rc1 is a ConstRef, because only the underlying value
    // of references is compared.
    expect(r1 == rc1, true);

    // true, because r1 and rc1 are a reference to the same
    // String Object. It does not matter that r1 is a MutableRef
    // and rc1 is a ConstRef, because only the underlying value
    // of references is used for calculating the hashCode.
    expect(r1.hashCode == rc1.hashCode, true);
  });

  test("Check reference parameters for functions.", () {
    void modifyInt(MutableRef<int> i) {
      i.value = 10;
    }

    final ref = 0.ref<int>();

    // 0, because we initialized it with this value.
    expect(ref.value, 0);

    modifyInt(ref);

    //  10, because modifyInt sets the value of ref to 10.
    expect(ref.value, 10);
  });
}
